Attribute VB_Name = "modSonyIR"
'RemotePRO - emulate IR remote controls using "simplest" IR circuit
'See circuit at http://www.geocities.com/odednoam for circuit
'Copyright (C) 2002 Oded Noam
'
'This program is free software; you can redistribute it and/or
'modify it under the terms of the GNU General Public License
'as published by the Free Software Foundation; either version 2
'of the License, or (at your option) any later version.
'
'This program is distributed in the hope that it will be useful,
'but WITHOUT ANY WARRANTY; without even the implied warranty of
'MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
'GNU General Public License for more details.
'
'You should have received a copy of the GNU General Public License
'along with this program; if not, write to the Free Software
'Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
Option Explicit

Private s_alBitMasks(1 To 32) As Long
Private bInitialized As Boolean

Public Enum ControlCommands
    ircMDEject = 500015022
    ircMDNext = 500015033
    ircMDPrev = 500015032
    ircmdplay = 500015042
    ircMDStop = 500015040
    ircmdpause = 500015041
    ircMDRec = 500015045
    ircMDEdit = 500015060
    ircMDPlay0 = 500015000
    ircMDScroll = 500015025
    ircTunerFrPlus = 500013018
    ircTunerFrMinus = 500013019
    ircTunerPlus = 500013016
    ircTunerMinus = 500013017
    ircTunerBand = 500013015
    ircTunerStereoMono = 500013033
    ircCDPlay = 500017050
    ircCDPause = 500017057
    ircCDStop = 500017056
    ircCDEject = 500017022
    ircCDPrev = 500016048
    ircCDNext = 500016049
    ircSysFunction = 500012105
    ircSysPwrOff = 500016047
    ircSysPwrToggle = 500016021
    ircSysSleep = 500016096
    ircSysYes = 1301850089
    ircSysNo = 500015061
    ircSysVolUp = 500016018
    ircSysVolDown = 500016019
    ircSysEdit = 1301850088
    ircSysDisplay = 500016075
    ircSysDBFB = 500012049
    ircSysClockTimerSelect = 500016098
    ircSysClockTimerSet = 500016101
    ircSysMusicMenu = 500012098
    ircSysRepeatStMo = 1301850045
    ircSysPlayMode = 1301850040
    ircSysPlus = 1301850120
    ircSysMinus = 1301850121
    ircSysCursorPrev = 1301850122
    ircSysCursorNext = 1301850123
    ircMDTrack1 = 500015000
    ircMDTrack2 = 500015001
    ircMDTrack3 = 500015002
    ircMDTrack4 = 500015003
    ircMDTrack5 = 500015004
    ircMDTrack6 = 500015005
    ircMDTrack7 = 500015006
    ircMDTrack8 = 500015007
    ircMDTrack9 = 500015008
    ircMDTrack0 = 500015009
    ircMDTrackMinus = 500015010

End Enum

Private Const c_iKeyboardID = 3130
Private Const c_dMinimumDelay As Double = (1 / 2)
Private s_fLastCommandTime As Single

Public Sub SendCommandE(in_eCommand As ControlCommands)
    SendCommand (in_eCommand Mod 1000), _
                Int(in_eCommand / 1000) Mod 100000, _
                Int(in_eCommand / 100000000)
End Sub

Public Sub SendString(in_strToSend As String)
    Dim i As Integer
    For i = 1 To Len(in_strToSend)
        SendCommand Asc(Mid(in_strToSend, i)), c_iKeyboardID, 13
    Next i
End Sub

Public Sub SendCommand(iCommand As Integer, _
                       iDeviceId As Integer, _
                       iDeviceIdLen As Integer)
    If (s_fLastCommandTime > Timer) Then s_fLastCommandTime = 0
        
    Dim strCommand As String
    Dim comm As New clsCommHandler
    Dim rc As Integer
    
    strCommand = ModulateCommand(CodeCommand(iDeviceId, iDeviceIdLen, iCommand, 7))
        Screen.MousePointer = vbHourglass
        
   Debug.Print "OpenPort rc " & rc
    Dim iFirstTry As Single
    iFirstTry = Timer
    Do
        rc = comm.OpenPort(2)
        If Not rc Then
            comm.ClosePort
            If Timer > iFirstTry + 10 Then
                Screen.MousePointer = vbDefault
                MsgBox ("Cannot open port")
                Exit Sub
            End If
            DoEvents
        End If
    Loop While Not rc
    
    rc = comm.SetState(115200, PARITY_NO, 8, STOP_1)
    Debug.Print "SetState rc " & rc
    If Not rc Then
        comm.ClosePort
        Screen.MousePointer = vbDefault
        MsgBox ("Cannot set comm state")
    End If
    
    While s_fLastCommandTime + c_dMinimumDelay > Timer
        'DoEvents
    Wend
    comm.CommWrite (strCommand)
    s_fLastCommandTime = Timer
    Debug.Print "CommWrite rc " & rc
    comm.ClosePort
    Screen.MousePointer = vbDefault
    
                           
End Sub


Private Function modCodingFunctions_Init()
    bInitialized = True
    Dim i As Integer
    Dim j As Long
    j = 1
    For i = 1 To 32
        s_alBitMasks(i) = j
        If i < 31 Then j = j * 2
    Next i
End Function

Private Function CodeCommand(iId As Integer, iIdBits As Integer, iCommand As Integer, iCommandBits As Integer) As String
    If Not bInitialized Then
        modCodingFunctions_Init
    End If
    ' Only 20 LSBs are counted
    Dim strCommand As String
    Dim i As Integer
    
    ' Command begins with a 2.4ms start bit 1 and 0.4ms 0
    strCommand = "11110"
    
    For i = 1 To iCommandBits
        If ((iCommand And s_alBitMasks(i)) <> 0) Then
            ' Code 1
            strCommand = strCommand & "110"
            Else
            ' Code 0
            strCommand = strCommand & "10"
        End If
    Next i
    
    For i = 1 To iIdBits
        If ((iId And s_alBitMasks(i)) <> 0) Then
            ' Code 1
            strCommand = strCommand & "110"
            Else
            ' Code 0
            strCommand = strCommand & "10"
        End If
    Next i
    
    ' Command ends with 0's up to 45 ms
    Dim nTailLength As Integer
    nTailLength = Int((45 / 0.57) - Len(strCommand))
    For i = 0 To nTailLength
        strCommand = strCommand & "0"
    Next i

    CodeCommand = strCommand
End Function

Private Function ModulateCommand(ByVal in_strCodedCommand) As String

    Dim strCommand As String
    Dim strZero As String
    Dim strOne As String
    Dim i As Integer

    If Not bInitialized Then
        MsgBox "modCodingFunctions not initialized"
        Exit Function
    End If

    ' Code each bit in the input to 0.4ms signal of 1 or of 40khz carrier
    ' bit rate mode is 115200, so for 38KHz carrier we do each byte as '1011011' (91) or '0010010' (18).
    ' (with start bit 1 and stop bit 0, its actual '1' '1011011' '0')
    For i = 0 To 6
        strZero = strZero & Chr(255)
        strOne = strOne & Chr(91) '91 for 101
    Next i

    For i = 1 To Len(in_strCodedCommand)
        If (Mid(in_strCodedCommand, i, 1) = "0") Then
           strCommand = strCommand & strZero
        Else
           strCommand = strCommand & strOne
        End If
    Next i

    ModulateCommand = strCommand & strCommand & strCommand & strCommand
    
End Function

Public Sub Delay(in_lTimeoutMs As Long)
    Dim DelayUntil As Double
    Dim DelayTime As Double
    DelayTime = (in_lTimeoutMs / 1000)
    DelayUntil = Timer + DelayTime
    While DelayUntil > Timer
        DoEvents
    Wend
End Sub
